#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/7/18 15:28
@File: test_env.py
@IDE: pycharm
@Description:
    测试swanlab.env模块
"""
import os

import pytest

import swanlab
from swanlab.env import SwanLabEnv, is_interactive, get_save_dir, remove_host_suffix


def test_set_default():
    """
    测试获取默认的环境变量
    """
    del os.environ[SwanLabEnv.WEB_HOST.value]
    del os.environ[SwanLabEnv.API_HOST.value]
    del os.environ[SwanLabEnv.RUNTIME.value]
    swanlab.env.SwanLabEnv.set_default()
    assert swanlab.package.get_host_web() == "https://swanlab.cn"
    assert swanlab.package.get_host_api() == "https://api.swanlab.cn/api"
    assert os.getenv(SwanLabEnv.RUNTIME.value) == "user"


def test_set_by_netrc():
    """
    测试通过netrc文件设置环境变量
    """
    del os.environ[SwanLabEnv.WEB_HOST.value]
    del os.environ[SwanLabEnv.API_HOST.value]
    del os.environ[SwanLabEnv.RUNTIME.value]
    netrc_path = os.path.join(get_save_dir(), ".netrc")
    with open(netrc_path, "w") as f:
        f.write("machine https://example.ai\nlogin test\npassword 123")
    swanlab.env.SwanLabEnv.set_default()
    assert swanlab.package.get_host_web() == "https://example.ai"
    assert swanlab.package.get_host_api() == "https://example.ai/api"
    assert os.getenv(SwanLabEnv.RUNTIME.value) == "user"


def test_check():
    """
    测试检查环境变量
    """
    os.environ[SwanLabEnv.MODE.value] = "124345"
    with pytest.raises(ValueError):
        SwanLabEnv.check()
    os.environ[SwanLabEnv.RUNTIME.value] = "124"
    with pytest.raises(ValueError):
        SwanLabEnv.check()


def test_is_interactive():
    # 测试时默认返回true
    assert is_interactive() == True


# ---------------------------------- 测试移除 host 后缀 ----------------------------------
def test_removes_suffix_when_host_ends_with_suffix():
    host = "example.ai/api"
    suffix = "/api"
    assert remove_host_suffix(host, suffix) == "example.ai"


def test_returns_original_host_when_host_does_not_end_with_suffix():
    host = "example.com"
    suffix = "/api"
    assert remove_host_suffix(host, suffix) == "example.com"


def test_handles_empty_host_and_returns_empty():
    host = ""
    suffix = "/api"
    assert remove_host_suffix(host, suffix) == ""


def test_handles_empty_suffix_and_returns_original_host():
    host = "example.com/api"
    suffix = ""
    assert remove_host_suffix(host, suffix) == "example.com/api"


def test_handles_suffix_longer_than_host_and_returns_original_host():
    host = "api"
    suffix = "example.com/api"
    assert remove_host_suffix(host, suffix) == "api"


def test_handles_host_with_multiple_suffixes():
    host = "example.com/api/v1"
    suffix = "/api/v1"
    assert remove_host_suffix(host, suffix) == "example.com"


def test_handles_host_with_blank_suffix():
    host = "example.com  "
    suffix = "api"
    assert remove_host_suffix(host, suffix) == "example.com"



"""
@author: caddiesnew
@file: metrics.py
@time: 2025-07-27 19:22:45
@description: 基于 echarts 实现部分 metrics 图表
"""

import importlib.util

import numpy as np
import pyecharts.options as opts
from pyecharts.charts import HeatMap, Line


def _check_sklearn():
    """Check if scikit-learn is installed."""
    if importlib.util.find_spec("sklearn") is None:
        raise TypeError("scikit-learn is required for this function. Please install it via `pip install scikit-learn`.")


def roc_curve(ground_truth, predictions, title=None) -> Line:
    """Plot Receiver operating characteristic (ROC) with AUC

    Parameters
    ----------
    ground_truth : ArrayLike. array-like of shape (n_samples,)
        True binary labels.
    predictions : ArrayLike. array-like of shape (n_samples,)
        Target scores, can either be probability estimates of the positive
            class, confidence values, or non-thresholded measure of decisions
    title : str or bool, optional
        Title for the chart. If True, will use default title with AUC.
        If string, will use that as title. If None or False, no title.
    """
    _check_sklearn()
    from sklearn import metrics

    fpr, tpr, _ = metrics.roc_curve(ground_truth, predictions)
    roc_auc = metrics.auc(fpr, tpr)

    # Diagonostic line
    diag_arr = fpr.copy()

    line = (
        Line()
        .add_xaxis(xaxis_data=fpr.tolist())
        .add_yaxis(
            series_name="TPR",
            y_axis=tpr.tolist(),
            is_smooth=True,
            is_symbol_show=False,
            label_opts=opts.LabelOpts(is_show=False),
            linestyle_opts=opts.LineStyleOpts(width=2),
        )
        .add_yaxis(
            series_name="",
            y_axis=diag_arr.tolist(),
            is_symbol_show=False,
            label_opts=opts.LabelOpts(is_show=False),
            linestyle_opts=opts.LineStyleOpts(width=2, type_="dashed"),
        )
    )

    # 处理标题
    if title is True:
        title_opts = opts.TitleOpts(title=f"ROC Curve (AUC = {roc_auc:.2f})", pos_left="center")
    elif isinstance(title, str):
        title_opts = opts.TitleOpts(title=title, pos_left="center")
    else:
        title_opts = None

    # 设置全局选项
    global_opts = {
        "xaxis_opts": opts.AxisOpts(type_="value", name="FPR"),
        "yaxis_opts": opts.AxisOpts(type_="value", name="TPR"),
        "legend_opts": opts.LegendOpts(is_show=False),
    }

    if title_opts:
        global_opts["title_opts"] = title_opts

    line.set_global_opts(**global_opts)
    return line


def pr_curve(ground_truth, predictions, title=None) -> Line:
    """Plot Precision-Recall(PR) Curve and AUC.

    Parameters
    ----------
    ground_truth : ArrayLike. array-like of shape (n_samples,)
        True binary labels.
    predictions : ArrayLike. array-like of shape (n_samples,)
        Target scores, can either be probability estimates of the positive
            class, confidence values, or non-thresholded measure of decisions
    title : str or bool, optional
        Title for the chart. If True, will use default title with AUC.
        If string, will use that as title. If None or False, no title.
    """
    _check_sklearn()
    from sklearn import metrics

    precision, recall, _ = metrics.precision_recall_curve(ground_truth, predictions)
    pr_auc = metrics.auc(recall, precision)

    line = (
        Line()
        .add_xaxis(xaxis_data=recall.tolist())
        .add_yaxis(
            series_name="Precision",
            y_axis=precision.tolist(),
            is_smooth=True,
            is_symbol_show=False,
            label_opts=opts.LabelOpts(is_show=False),
            linestyle_opts=opts.LineStyleOpts(width=2),
        )
    )

    # 处理标题
    if title is True:
        title_opts = opts.TitleOpts(title=f"PR Curve (AUC = {pr_auc:.2f})", pos_left="center")
    elif isinstance(title, str):
        title_opts = opts.TitleOpts(title=title, pos_left="center")
    else:
        title_opts = None

    # 设置全局选项
    global_opts = {
        "xaxis_opts": opts.AxisOpts(type_="value", name="Recall"),
        "yaxis_opts": opts.AxisOpts(type_="value", name="Precision"),
        "legend_opts": opts.LegendOpts(is_show=False),
    }

    if title_opts:
        global_opts["title_opts"] = title_opts

    line.set_global_opts(**global_opts)
    return line


def confusion_matrix(ground_truth, predictions, class_names, title=None) -> HeatMap:
    """Compute and Plot Confusion Matrix

    Parameters
    ----------
        ground_truth: ArrayLike: Ground truth (correct) target values.
        predictions: ArrayLike: Estimated targets as returned by a classifier.
        class_names: (list[str]): class names
        title : str or bool, optional
            Title for the chart. If True, will use default title.
            If string, will use that as title. If None or False, no title.
    """
    _check_sklearn()
    from sklearn import metrics

    cm = metrics.confusion_matrix(ground_truth, predictions)
    # normalized
    cm = cm.astype("float") / cm.sum(axis=1)[:, np.newaxis]
    # 注意：pyecharts中[x, y, value]的x对应列(预测)，y对应行(真实)
    # pyecharts的y轴是从下往上，而混淆矩阵是从上往下，需要反转y坐标
    data = []
    for i in range(len(class_names)):
        for j in range(len(class_names)):
            data.append([class_names[i], class_names[j], cm[i][j]])

    heatmap = (
        HeatMap()
        .add_xaxis(class_names)
        .add_yaxis(
            series_name="Confusion Matrix",
            yaxis_data=class_names,  # type: ignore
            value=data,
            label_opts=opts.LabelOpts(is_show=True, position="inside"),
        )
    )

    # 处理标题
    if title is True:
        title_opts = opts.TitleOpts(title="Confusion Matrix", pos_left="center")
    elif isinstance(title, str):
        title_opts = opts.TitleOpts(title=title, pos_left="center")
    else:
        title_opts = None

    # 设置全局选项
    global_opts = {
        "xaxis_opts": opts.AxisOpts(
            name="Prediction",
            type_="category",
            splitline_opts=opts.SplitLineOpts(
                is_show=True,
                linestyle_opts=opts.LineStyleOpts(color="#fff", width=2),
            ),
        ),
        "yaxis_opts": opts.AxisOpts(
            name="Ground truth",
            type_="category",
            splitline_opts=opts.SplitLineOpts(
                is_show=True,
                linestyle_opts=opts.LineStyleOpts(color="#fff", width=2),
            ),
        ),
        "legend_opts": opts.LegendOpts(is_show=False),
        "visualmap_opts": opts.VisualMapOpts(
            pos_right="right",
            pos_top="center",
            orient="vertical",
            min_=0,
            max_=int(cm.max()),
            is_piecewise=False,
            # 使用 seaborn 默认的 colormap
            range_color=[
                "#f7fbff",
                "#deebf7",
                "#c6dbef",
                "#9ecae1",
                "#6baed6",
                "#4292c6",
                "#2171b5",
                "#08519c",
                "#08306b",
            ],
        ),
    }

    if title_opts:
        global_opts["title_opts"] = title_opts

    heatmap.set_global_opts(**global_opts)
    return heatmap



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: swanlab/proto/common/v1/common.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'swanlab/proto/common/v1/common.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$swanlab/proto/common/v1/common.proto\x12\x17swanlab.proto.common.v1\"\x80\x02\n\x08\x41nyValue\x12\x16\n\x0cstring_value\x18\x01 \x01(\tH\x00\x12\x14\n\nbool_value\x18\x02 \x01(\x08H\x00\x12\x13\n\tint_value\x18\x03 \x01(\x03H\x00\x12\x16\n\x0c\x64ouble_value\x18\x04 \x01(\x01H\x00\x12:\n\x0b\x61rray_value\x18\x05 \x01(\x0b\x32#.swanlab.proto.common.v1.ArrayValueH\x00\x12=\n\x0ckvlist_value\x18\x06 \x01(\x0b\x32%.swanlab.proto.common.v1.KeyValueListH\x00\x12\x15\n\x0b\x62ytes_value\x18\x07 \x01(\x0cH\x00\x42\x07\n\x05value\"?\n\nArrayValue\x12\x31\n\x06values\x18\x01 \x03(\x0b\x32!.swanlab.proto.common.v1.AnyValue\"A\n\x0cKeyValueList\x12\x31\n\x06values\x18\x01 \x03(\x0b\x32!.swanlab.proto.common.v1.KeyValue\"I\n\x08KeyValue\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x30\n\x05value\x18\x02 \x01(\x0b\x32!.swanlab.proto.common.v1.AnyValueB\rZ\x0b\x63ore/pkg/pbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'swanlab.proto.common.v1.common_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\013core/pkg/pb'
  _globals['_ANYVALUE']._serialized_start=66
  _globals['_ANYVALUE']._serialized_end=322
  _globals['_ARRAYVALUE']._serialized_start=324
  _globals['_ARRAYVALUE']._serialized_end=387
  _globals['_KEYVALUELIST']._serialized_start=389
  _globals['_KEYVALUELIST']._serialized_end=454
  _globals['_KEYVALUE']._serialized_start=456
  _globals['_KEYVALUE']._serialized_end=529
# @@protoc_insertion_point(module_scope)



"""
@author: cunyue
@file: model.py
@time: 2025/6/16 14:12
@description: 定义上传模型
"""

import json
from datetime import datetime
from enum import Enum
from typing import List, Optional, TypedDict, Literal

from swanlab.data.modules import MediaBuffer
from swanlab.toolkit import ColumnClass, ColumnConfig, LogContent


class ColumnModel:
    """
    列信息上传模型
    """

    def __init__(
        self,
        key,
        name: Optional[str],
        cls: ColumnClass,
        typ: str,
        config: Optional[ColumnConfig],
        section_name: Optional[str],
        section_type: Optional[str],
        error: dict = None,
    ):
        """
        key: 键
        name: 键的名称
        cls: 键的类别
        typ: 键的类型
        config: 键的配置
        section_name: 键所在的section的名称
        section_type: 键所在的section的类型
        error: 错误信息
        """
        self.key = key
        self.name = name
        self.cls = cls
        self.typ = typ
        self.config = config
        self.section_name = section_name
        self.section_type = section_type
        self.error = error

    def to_dict(self):
        """
        序列化为Dict,传递给后端
        """
        d = {
            "class": self.cls,
            "type": self.typ,
            "key": self.key,
            "name": self.name,
            "error": self.error,
            "sectionName": self.section_name,
            "sectionType": self.section_type,
        }
        if self.name is None:
            d.pop("name")
        if self.error is None:
            d.pop("error")
        if self.section_name is None:
            d.pop("sectionName")
        if self.section_type is None:
            d.pop("sectionType")
        if self.config is None:
            return d
        # 将额外的图表配置信息加入
        if self.config.y_range is not None:
            d["yRange"] = self.config.y_range
        if self.config.chart_name is not None:
            d["chartName"] = self.config.chart_name
        if self.config.chart_index is not None:
            d["chartIndex"] = self.config.chart_index
        if self.config.metric_name is not None:
            d["metricName"] = self.config.metric_name
        if self.config.metric_color is not None:
            d["metricColors"] = self.config.metric_color
        return d


class MetricType(Enum):
    """
    指标类型枚举
    """

    SCALAR = "scalar"
    """
    标量指标
    """
    MEDIA = "media"
    """
    媒体指标
    """
    LOG = "log"
    """
    日志数据
    """


class MediaModel:
    """
    媒体指标信息上传模型
    """

    type = MetricType.MEDIA

    def __init__(
        self,
        metric: dict,
        key: str,
        key_encoded: str,
        step: int,
        epoch: int,
        buffers: List[MediaBuffer] = None,
    ):

        # -------------------------- 🤡这里是一点小小的💩 --------------------------
        # 要求上传时的文件路径必须带key_encoded前缀
        if buffers is not None:
            metric = json.loads(json.dumps(metric))
            for i, d in enumerate(metric["data"]):
                metric["data"][i] = "{}/{}".format(key_encoded, d)
        # ------------------------------------------------------------------------

        self.metric = metric
        self.step = step
        self.epoch = epoch
        self.key = key
        """
        真实的指标名称
        """
        self.key_encoded = key_encoded
        """
        编码后路径安全的指标名称
        """
        self.buffers = buffers
        """
        原始数据，可能为None
        """

    def to_dict(self):
        """
        序列化
        """
        return {
            **self.metric,
            "key": self.key,
            "index": self.step,
            "epoch": self.epoch,
        }


class ScalarModel:
    """
    标量指标信息上传模型
    """

    type = MetricType.SCALAR

    def __init__(self, metric: dict, key: str, step: int, epoch: int):
        self.metric = metric
        self.key = key
        self.step = step
        self.epoch = epoch

    def to_dict(self):
        """
        序列化
        """
        return {
            **self.metric,
            "key": self.key,
            "index": self.step,
            "epoch": self.epoch,
        }


class FileModel:
    """
    运行时文件信息上传模型
    """

    def __init__(
        self,
        requirements: str = None,
        metadata: dict = None,
        config: dict = None,
        conda: str = None,
    ):
        self.requirements = requirements
        self.metadata = metadata
        self.config = config
        self.conda = conda
        # 主要用于去重，保留最新的文件
        self.create_time = datetime.now()

    @classmethod
    def create(cls, file_models: List["FileModel"]) -> "FileModel":
        """
        比较若干个FileModel，获取最新的FileModel，并且保证其内部属性不为None
        """
        # 按照时间排序，倒叙排列，这意味着最新的排在第一个
        file_models = sorted(file_models, key=lambda x: x.create_time, reverse=True)
        lr, lm, lc, lo = None, None, None, None
        for file_model in file_models:
            lr = file_model.requirements if lr is None else lr
            lm = file_model.metadata if lm is None else lm
            lc = file_model.config if lc is None else lc
            lo = file_model.conda if lo is None else lo
            if lr is not None and lm is not None and lc is not None and lo is not None:
                break
        return FileModel(lr, lm, lc, lo)

    def to_dict(self):
        """
        序列化，会删除为None的字段
        """
        d = {
            "requirements": self.requirements,
            "metadata": self.metadata,
            "config": self.config,
            "conda": self.conda,
        }
        return {k: v for k, v in d.items() if v is not None}

    @property
    def empty(self):
        """
        是否为空
        """
        return all(getattr(self, attr) is None for attr in ('requirements', 'metadata', 'config', 'conda'))


class LogModel(TypedDict):
    """
    日志信息上传模型
    """

    level: Literal['INFO', 'WARN', 'ERROR']
    """
    日志级别
    """
    contents: List[LogContent]
    """
    当前日志级别的日志内容
    """


__all__ = [
    "LogModel",
    "ColumnModel",
    "ScalarModel",
    "MediaModel",
    "FileModel",
]
